// $Id: LOCA_Bifurcation_Factory.C,v 1.1.2.3 2005/08/18 23:12:38 etphipp Exp $
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_Bifurcation_Factory.C,v $

//@HEADER
// ************************************************************************
//
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
//
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Andy Salinger (agsalin@sandia.gov) or Eric Phipps
// (etphipp@sandia.gov), Sandia National Laboratories.
//
// ************************************************************************
//@HEADER

#include "NOX_Parameter_List.H"
#include "LOCA_GlobalData.H"
#include "LOCA_ErrorCheck.H"

#include "LOCA_Bifurcation_Factory.H"
#include "LOCA_TurningPoint_MooreSpence_ExtendedGroup.H"
#include "LOCA_TurningPoint_MooreSpence_AbstractGroup.H"

LOCA::Bifurcation::Factory::Factory(
	        const Teuchos::RefCountPtr<LOCA::GlobalData>& global_data) : 
  globalData(global_data)
{
}

LOCA::Bifurcation::Factory::~Factory()
{
}

Teuchos::RefCountPtr<LOCA::MultiContinuation::AbstractGroup>
LOCA::Bifurcation::Factory::create(
      const Teuchos::RefCountPtr<LOCA::Parameter::SublistParser>& topParams,
      const Teuchos::RefCountPtr<NOX::Parameter::List>& bifurcationParams,
      const Teuchos::RefCountPtr<LOCA::MultiContinuation::AbstractGroup>& grp)
{
  string methodName = "LOCA::Bifurcation::Factory::create()";
  Teuchos::RefCountPtr<LOCA::MultiContinuation::AbstractGroup> strategy;

  // Get name of strategy
  const string& name = strategyName(*bifurcationParams);

  if (name == "None")
    strategy = grp;

  else if (name == "Turning Point:  Moore-Spence") {

    // Cast group to MooreSpence group
    Teuchos::RefCountPtr<LOCA::TurningPoint::MooreSpence::AbstractGroup> msg = 
      Teuchos::rcp_dynamic_cast<LOCA::TurningPoint::MooreSpence::AbstractGroup>(grp);
    if (msg.get() == NULL)
      globalData->locaErrorCheck->throwError(
		    methodName,
		    string("Underlying group must be derived from ") + 
		    string("LOCA::TurningPoint::MooreSpence::AbstractGroup ") +
		    string("for Moore-Spence turning point continuation!"));

    strategy = 
      Teuchos::rcp(new LOCA::TurningPoint::MooreSpence::ExtendedGroup(
							   globalData,
							   topParams,
							   bifurcationParams,
							   msg));
  }
  else if (name == "User-Defined") {

    // Get name of user-defined strategy
    string userDefinedName = bifurcationParams->getParameter(
							 "User-Defined Name",
							 "???");
    if ((*bifurcationParams).INVALID_TEMPLATE_QUALIFIER
	  isParameterRcp<LOCA::MultiContinuation::AbstractGroup>(userDefinedName))
      strategy = (*bifurcationParams).INVALID_TEMPLATE_QUALIFIER
	getRcpParameter<LOCA::MultiContinuation::AbstractGroup>(userDefinedName);
    else
       globalData->locaErrorCheck->throwError(
				       methodName,
				       "Cannot find user-defined strategy: " + 
				       userDefinedName);
  }
  else
    globalData->locaErrorCheck->throwError(
				      methodName,
				      "Invalid bifurcation method: " + 
				      name);

  return strategy;
}

const string&
LOCA::Bifurcation::Factory::strategyName(
				NOX::Parameter::List& bifurcationParams) const
{
  return bifurcationParams.getParameter("Method", "None");
}
