// $Id: LOCA_Continuation_Manager.C,v 1.10 2005/02/01 19:33:41 mnphenow Exp $ 
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/Attic/LOCA_Continuation_Manager.C,v $ 

//@HEADER
// ************************************************************************
// 
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//                                                                                 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA                                                                                
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov) or Roger Pawlowski
// (rppawlo@sandia.gov), Sandia National Laboratories.
// 
// ************************************************************************
//@HEADER

#include "NOX_Parameter_List.H"
#include "LOCA_Continuation_Manager.H"
#include "LOCA_Continuation_AbstractGroup.H"
#include "LOCA_Continuation_ExtendedGroup.H"
#include "LOCA_Continuation_NaturalGroup.H"
#include "LOCA_Continuation_ArcLengthGroup.H"
#include "LOCA_Continuation_HouseholderGroup.H"
#include "LOCA_Utils.H"

LOCA::Continuation::Manager::Manager(NOX::Parameter::List& p) :
  method(),
  conParamID(),
  paramsPtr(NULL)
{
  reset(p);
}

LOCA::Continuation::Manager::~Manager()
{
}

NOX::Abstract::Group::ReturnType 
LOCA::Continuation::Manager::reset(NOX::Parameter::List& p) 
{
  method = p.getParameter("Continuation Method", "Natural");
  conParamID = p.getParameter("Continuation Parameter", "???");
  paramsPtr = &p;
  return NOX::Abstract::Group::Ok;
}

LOCA::Continuation::ExtendedGroup* 
LOCA::Continuation::Manager::createContinuationGroup(
			            LOCA::Continuation::AbstractGroup& grp) 
{
  if (method == "Natural") 
    return new LOCA::Continuation::NaturalGroup(grp, conParamID, *paramsPtr);
  else if (method == "Arc Length")
    return new LOCA::Continuation::ArcLengthGroup(grp, conParamID, *paramsPtr);
  else if (method == "Householder Arc Length")
    return new LOCA::Continuation::HouseholderGroup(grp, conParamID, 
						    *paramsPtr);
  else {
    if (LOCA::Utils::doPrint(LOCA::Utils::Error)) {
      cout << "LOCA::Continuation::Manager::createContinuationGroup() "
	   << "- invalid choice (" << method 
	   << ") for continuation method " << endl;
    }
    throw "LOCA Error";
    return NULL;
  }
}

const string&
LOCA::Continuation::Manager::getMethod() const 
{
  return method;
}

const string&
LOCA::Continuation::Manager::getConParamID() const 
{
  return conParamID;
}
