// $Id: LOCA_GlobalData.H,v 1.1 2005/03/15 00:14:04 etphipp Exp $
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_GlobalData.H,v $

//@HEADER
// ************************************************************************
//
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
//
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Andy Salinger (agsalin@sandia.gov) or Eric Phipps
// (etphipp@sandia.gov), Sandia National Laboratories.
//
// ************************************************************************
//@HEADER

#ifndef LOCA_GLOBALDATA_H
#define LOCA_GLOBALDATA_H

#include "Teuchos_RefCountPtr.hpp"

// Global data objects forward declarations
namespace LOCA {
  class Utils;
  class ErrorCheck;
  class Factory;
}

namespace LOCA {

  //! Container class to hold "global" %LOCA objects
  /*!
   * %GlobalData is a container class that holds ref-count pointers to
   * "global" objects, i.e., objects that nearly every LOCA object will need
   * access to.  By putting them all in one container class, the container
   * class can be stored in each LOCA object, and if a new global object
   * is needed, it can be added here without modifying the rest of the
   * code.  This is an alternative to true global or static objects which are
   * note safe in many contexts.  In particular, this approach allows multiple
   * %LOCA "invocations" to be in memory at the same time. Note that all data 
   * members are declared public.
   */
  class GlobalData {

  public:

    //! Constructor taking a ref-count pointer to each global object
    GlobalData(const Teuchos::RefCountPtr<LOCA::Utils>& loca_utils,
	       const Teuchos::RefCountPtr<LOCA::ErrorCheck>& loca_error_check,
	       const Teuchos::RefCountPtr<LOCA::Factory>& loca_factory);

    //! Destructor
    virtual ~GlobalData();

    //! Ref-count pointer to LOCA::Utils object
    Teuchos::RefCountPtr<LOCA::Utils> locaUtils;

    //! Ref-count pointer to LOCA::ErrorCheck object
    Teuchos::RefCountPtr<LOCA::ErrorCheck> locaErrorCheck;

    //! Ref-count pointer to LOCA::Factory object
    Teuchos::RefCountPtr<LOCA::Factory> locaFactory;

  private:

    //! Copy constructor is private to preclude copying
    GlobalData(const GlobalData&);

    //! Assignment operator is private to preclude copying
    GlobalData& operator = (const GlobalData&);

  }; // Class GlobalData

} // Namespace LOCA

#endif
