// $Id: LOCA_Parameter_SublistParser.C,v 1.3 2005/05/03 21:38:47 etphipp Exp $
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_Parameter_SublistParser.C,v $

//@HEADER
// ************************************************************************
//
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
//
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Andy Salinger (agsalin@sandia.gov) or Eric Phipps
// (etphipp@sandia.gov), Sandia National Laboratories.
//
// ************************************************************************
//@HEADER

#include "NOX_Parameter_List.H"
#include "LOCA_Parameter_SublistParser.H"
#include "LOCA_GlobalData.H"
#include "LOCA_ErrorCheck.H"

LOCA::Parameter::SublistParser::SublistParser(
		  const Teuchos::RefCountPtr<LOCA::GlobalData>& global_data) :
  globalData(global_data),
  sublistMap()
{
}

LOCA::Parameter::SublistParser::~SublistParser()
{
}

void
LOCA::Parameter::SublistParser::parseSublists(
	     const Teuchos::RefCountPtr<NOX::Parameter::List>& topLevelParams)
{
  // Top level sublist
  sublistMap["Top Level"] = topLevelParams;

  // LOCA sublist
  NOX::Parameter::List& locaSublist = topLevelParams->sublist("LOCA");
  sublistMap["LOCA"] = Teuchos::rcp(&locaSublist, false);

  // Stepper sublist
  NOX::Parameter::List& stepperSublist = locaSublist.sublist("Stepper");
  sublistMap["Stepper"] = Teuchos::rcp(&stepperSublist, false);

  // Eigensolver sublist
  NOX::Parameter::List& eigensolverSublist = 
    stepperSublist.sublist("Eigensolver");
  sublistMap["Eigensolver"] = Teuchos::rcp(&eigensolverSublist, false);

  // Constraints sublist
  NOX::Parameter::List& constraintsSublist = 
    locaSublist.sublist("Constraints");
  sublistMap["Constraints"] = Teuchos::rcp(&constraintsSublist, false);

  // Bifurcation sublist
  NOX::Parameter::List& bifurcationSublist = 
    locaSublist.sublist("Bifurcation");
  sublistMap["Bifurcation"] = Teuchos::rcp(&bifurcationSublist, false);

  // Predictor sublist
  NOX::Parameter::List& predictorSublist = locaSublist.sublist("Predictor");
  sublistMap["Predictor"] = Teuchos::rcp(&predictorSublist, false);

  // First Step Predictor sublist
  NOX::Parameter::List& fspredictorSublist = 
    predictorSublist.sublist("First Step Predictor");
  sublistMap["First Step Predictor"] = 
    Teuchos::rcp(&fspredictorSublist, false);

  // Last Step Predictor sublist
  NOX::Parameter::List& lspredictorSublist = 
    predictorSublist.sublist("Last Step Predictor");
  sublistMap["Last Step Predictor"] = Teuchos::rcp(&lspredictorSublist, false);

  // Stepsize sublist
  NOX::Parameter::List& stepsizeSublist = locaSublist.sublist("Step Size");
  sublistMap["Step Size"] = Teuchos::rcp(&stepsizeSublist, false);

  // Utilities sublist
  NOX::Parameter::List& utilitiesSublist = locaSublist.sublist("Utilities");
  sublistMap["Utilities"] = Teuchos::rcp(&utilitiesSublist, false);

  // NOX sublist
  NOX::Parameter::List& noxSublist = topLevelParams->sublist("NOX");
  sublistMap["NOX"] = Teuchos::rcp(&noxSublist, false);

  // Direction sublist
  NOX::Parameter::List& directionSublist = noxSublist.sublist("Direction");
  sublistMap["Direction"] = Teuchos::rcp(&directionSublist, false);

  // Newton sublist
  NOX::Parameter::List& newtonSublist = directionSublist.sublist("Newton");
  sublistMap["Newton"] = Teuchos::rcp(&newtonSublist, false);

  // Linear Solver sublist
  NOX::Parameter::List& lsSublist = newtonSublist.sublist("Linear Solver");
  sublistMap["Linear Solver"] = Teuchos::rcp(&lsSublist, false);

  // Line Search sublist
  NOX::Parameter::List& lineSearchSublist = noxSublist.sublist("Line Search");
  sublistMap["Line Search"] = Teuchos::rcp(&lineSearchSublist, false);

  // Printing sublist
  NOX::Parameter::List& printingSublist = noxSublist.sublist("Printing");
  sublistMap["Printing"] = Teuchos::rcp(&printingSublist, false);
}

Teuchos::RefCountPtr<NOX::Parameter::List> 
LOCA::Parameter::SublistParser::getSublist(const string& name)
{
  // Find name in list, if it exists.
  SublistMapIterator i = sublistMap.find(name);

  // If it does not exist throw an error.
  if (i == sublistMap.end()) {
   globalData->locaErrorCheck->throwError(
			     "LOCA::Parameter::SublistParser::getSublist()",
			     "Invalid sublist name: " + name);
  }

  // Return sublist
  return (*i).second;
}
