// $Id: LOCA_SaveEigenData_Factory.C,v 1.1.2.1 2005/08/23 19:33:39 etphipp Exp $
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_SaveEigenData_Factory.C,v $

//@HEADER
// ************************************************************************
//
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
//
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Andy Salinger (agsalin@sandia.gov) or Eric Phipps
// (etphipp@sandia.gov), Sandia National Laboratories.
//
// ************************************************************************
//@HEADER

#include "NOX_Parameter_List.H"
#include "LOCA_GlobalData.H"
#include "LOCA_ErrorCheck.H"

#include "LOCA_SaveEigenData_Factory.H"
#include "LOCA_SaveEigenData_AbstractStrategy.H"
#include "LOCA_SaveEigenData_DefaultStrategy.H"

LOCA::SaveEigenData::Factory::Factory(
	        const Teuchos::RefCountPtr<LOCA::GlobalData>& global_data) : 
  globalData(global_data)
{
}

LOCA::SaveEigenData::Factory::~Factory()
{
}

Teuchos::RefCountPtr<LOCA::SaveEigenData::AbstractStrategy>
LOCA::SaveEigenData::Factory::create(
       const Teuchos::RefCountPtr<LOCA::Parameter::SublistParser>& topParams,
       const Teuchos::RefCountPtr<NOX::Parameter::List>& eigenParams)
{
  string methodName = "LOCA::SaveEigenData::Factory::create()";
  Teuchos::RefCountPtr<LOCA::SaveEigenData::AbstractStrategy> strategy;

  // Get name of strategy
  const string& name = strategyName(*eigenParams);

  if (name == "Default")
    strategy = 
      Teuchos::rcp(new LOCA::SaveEigenData::DefaultStrategy(globalData,
							    topParams,
							    eigenParams));
  else if (name == "User-Defined") {

    // Get name of user-defined strategy
    string userDefinedName = eigenParams->getParameter(
					  "User-Defined Save Eigen Data Name",
					  "???");
    if ((*eigenParams).INVALID_TEMPLATE_QUALIFIER
	  isParameterRcp<LOCA::SaveEigenData::AbstractStrategy>(userDefinedName))
      strategy = (*eigenParams).INVALID_TEMPLATE_QUALIFIER
	getRcpParameter<LOCA::SaveEigenData::AbstractStrategy>(userDefinedName);
    else
       globalData->locaErrorCheck->throwError(
				       methodName,
				       "Cannot find user-defined strategy: " + 
				       userDefinedName);
  }
  else
    globalData->locaErrorCheck->throwError(
				      methodName,
				      "Invalid save eigen data strategy: " + 
				      name);

  return strategy;
}

const string&
LOCA::SaveEigenData::Factory::strategyName(
				  NOX::Parameter::List& eigenParams) const
{
  return eigenParams.getParameter("Save Eigen Data Method", "Default");
}
