// $Id: LOCA_SingularJacobianSolve_ItRef.H,v 1.4 2005/02/01 19:33:41 mnphenow Exp $ 
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_SingularJacobianSolve_ItRef.H,v $ 

//@HEADER
// ************************************************************************
// 
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//                                                                                 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA                                                                                
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov) or Roger Pawlowski
// (rppawlo@sandia.gov), Sandia National Laboratories.
// 
// ************************************************************************
//@HEADER

#ifndef LOCA_SINGULARJACOBIANSOLVE_ITREF_H
#define LOCA_SINGULARJACOBIANSOLVE_ITREF_H

#include "LOCA_SingularJacobianSolve_Generic.H"  // base class

namespace LOCA {

  namespace SingularJacobianSolve {

    /*! 
     * \brief This class computes the solution to \f$J x = b\f$ using 
     * one step of iterative refinement.
     */
    /*!
     * This singular solve method uses one step of iterative refinement to
     * improve the accuracy of the solution to the linear system 
     * \f$J x = b\f$.  In particular, the algorithm used here is
      * \f[
     *   \begin{aligned}
     *     &\text{Solve}\; Jx_1 = b \\
     *     &r = b - Jx_1 \\
     *     &\text{Solve}\; Jx_2 = r \\
     *     &x = x_1 + x_2
     *   \end{aligned}
     * \f]
     * Both solves use the underlying group's applyJacobianInverse() method 
     * and therefore this is a generic technique for computing solutions to 
     * nearly singular system since it uses any supplied linear solver.  

     * This algorithm is selected by setting the "Method" parameter of the 
     * "Singular Solve" sublist of the NOX linear solver parameter list to 
     * "Iterative Refinement".
     */
    class ItRef : public LOCA::SingularJacobianSolve::Generic {

    public:

      //! Constructor. 
      ItRef(NOX::Parameter::List& params);

      //! Copy constructor
      ItRef(const ItRef& source);
      
      //! Destructor
      virtual ~ItRef();

      //! Clone function
      virtual LOCA::SingularJacobianSolve::Generic* clone() const;

      //! Assignment operator
      virtual LOCA::SingularJacobianSolve::Generic& 
      operator = (const LOCA::SingularJacobianSolve::Generic& source);

      //! Assignment operator
      virtual ItRef&
      operator = (const ItRef& source);

      //! Reset parameters
      /*!
	There are no additional parameters for the Nic calculation.
      */
      virtual NOX::Abstract::Group::ReturnType 
      reset(NOX::Parameter::List& params);
  
      //! Computes the solution as described above.
      virtual NOX::Abstract::Group::ReturnType 
      compute(NOX::Parameter::List& params,
	      LOCA::Continuation::AbstractGroup& grp,
	      const NOX::Abstract::Vector& input,
	      const NOX::Abstract::Vector& approxNullVec,
	      const NOX::Abstract::Vector& jacApproxNullVec,
	      NOX::Abstract::Vector& result);

      //! Computes solution for multiple RHS 
      virtual NOX::Abstract::Group::ReturnType 
      computeMulti(NOX::Parameter::List& params,
		   LOCA::Continuation::AbstractGroup& grp,
		   const NOX::Abstract::Vector*const* inputs,
		   const NOX::Abstract::Vector& approxNullVec,
		   const NOX::Abstract::Vector& jacApproxNullVec,
		   NOX::Abstract::Vector** results,
		   int nVecs);

    };
  } // namespace SingularJacobianSolve
} // namespace LOCA

#endif
