// $Id: LOCA_StatusTest_Wrapper.H,v 1.5 2005/02/01 19:33:41 mnphenow Exp $ 
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_StatusTest_Wrapper.H,v $ 

//@HEADER
// ************************************************************************
// 
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//                                                                                 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA                                                                                
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov) or Roger Pawlowski
// (rppawlo@sandia.gov), Sandia National Laboratories.
// 
// ************************************************************************
//@HEADER

#ifndef LOCA_STATUSTEST_WRAPPER_H
#define LOCA_STATUSTEST_WRAPPER_H

#include "NOX_StatusTest_Generic.H" // base class

namespace LOCA {

  //! %LOCA %StatusTest classes
  namespace StatusTest {

    /*!
     * \brief A %wrapper class for wrapping a %NOX status test.
     */
    /*!
     * The %LOCA::StatusTest::Wrapper class provides a wrapper for %NOX
     * status tests to change the solver passed to the wrapped status test.
     * The solver passed through the checkStatus() method is wrapped via 
     * the LOCA::Solver::Wrapper class and then forwarded to the checkStatus()
     * method of the wrapped status test.  The purpose of this is to allow
     * status tests that use concrete group data to function correctly when
     * the group is stored in an extended continuation or bifurcation group.
     * (See LOCA::Solver::Wrapper for more details or the %LOCA status tests
     * page for examples on how to effectively use this class.)
     */
    class Wrapper : public NOX::StatusTest::Generic {

    public:

      //! Constructor
      Wrapper(NOX::StatusTest::Generic& s);

      //! Destructor
      virtual ~Wrapper();

      //! Calls checkStatus of underlying status test
      virtual NOX::StatusTest::StatusType 
      checkStatus(const NOX::Solver::Generic& problem);

      //! Calls checkStatus of underlying status test
      virtual NOX::StatusTest::StatusType
      checkStatusEfficiently(const NOX::Solver::Generic& problem, 
			     NOX::StatusTest::CheckType checkType);

      //! Calls getStatus of underlying status test
      virtual NOX::StatusTest::StatusType getStatus() const;
      
      //! Calls print of underlying status test
      virtual ostream& print(ostream& stream, int indent = 0) const;

      //! Returns underlying status test
      NOX::StatusTest::Generic&
      getUnderlyingStatusTest();

      //! Returns underlying status test
      const NOX::StatusTest::Generic&
      getUnderlyingStatusTest() const;
  
    protected:

      //! Stores pointer to underlying status test
      NOX::StatusTest::Generic *statusTestPtr;
      
    }; // class Wrapper

  } // namespace StatusTest
} // namespace LOCA


#endif
