// $Id: LOCA_TurningPoint_MooreSpence_ExtendedMultiVector.C,v 1.1.2.1 2005/08/03 23:59:43 etphipp Exp $ 
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_TurningPoint_MooreSpence_ExtendedMultiVector.C,v $ 

//@HEADER
// ************************************************************************
// 
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//                                                                                 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA                                                                                
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov) or Roger Pawlowski
// (rppawlo@sandia.gov), Sandia National Laboratories.
// 
// ************************************************************************
//@HEADER

#include "LOCA_TurningPoint_MooreSpence_ExtendedMultiVector.H" 
#include "LOCA_TurningPoint_MooreSpence_ExtendedVector.H"  

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::ExtendedMultiVector(
				       const NOX::Abstract::Vector& cloneVec,
				       int nColumns) :
  LOCA::Extended::MultiVector(nColumns, 2, 1)
{
  NOX::Abstract::MultiVector* mv1 = cloneVec.createMultiVector(nColumns, 
							      NOX::ShapeCopy);
  NOX::Abstract::MultiVector* mv2 = cloneVec.createMultiVector(nColumns, 
							      NOX::ShapeCopy);
  LOCA::Extended::MultiVector::setMultiVectorPtr(0, mv1);
  LOCA::Extended::MultiVector::setMultiVectorPtr(1, mv2);
}

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::ExtendedMultiVector(
		  const NOX::Abstract::MultiVector& xVec,
		  const NOX::Abstract::MultiVector& nullVec,
		  const NOX::Abstract::MultiVector::DenseMatrix& bifParams) :
  LOCA::Extended::MultiVector(xVec.numVectors(), 2, 1)
{
  LOCA::Extended::MultiVector::setMultiVectorPtr(0, xVec.clone(NOX::DeepCopy));
  LOCA::Extended::MultiVector::setMultiVectorPtr(1, 
						 nullVec.clone(NOX::DeepCopy));
  LOCA::Extended::MultiVector::getScalars() = bifParams;
}

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::ExtendedMultiVector(
	  const LOCA::TurningPoint::MooreSpence::ExtendedMultiVector& source, 
	  NOX::CopyType type) :
  LOCA::Extended::MultiVector(source, type)
{
}

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::ExtendedMultiVector(
	   const LOCA::TurningPoint::MooreSpence::ExtendedMultiVector& source, 
	   int nColumns) :
  LOCA::Extended::MultiVector(source, nColumns)
{
}

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::ExtendedMultiVector(
	   const LOCA::TurningPoint::MooreSpence::ExtendedMultiVector& source, 
	   const vector<int>& index, bool view) :
  LOCA::Extended::MultiVector(source, index, view)
{
}

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::~ExtendedMultiVector()
{
}

LOCA::Extended::MultiVector& 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::operator=(
					 const LOCA::Extended::MultiVector& y)
{
  return operator=(dynamic_cast<const LOCA::TurningPoint::MooreSpence::ExtendedMultiVector&>(y));
}

NOX::Abstract::MultiVector& 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::operator=(
					 const NOX::Abstract::MultiVector& y)
{
 return operator=(dynamic_cast<const LOCA::TurningPoint::MooreSpence::ExtendedMultiVector&>(y));
}

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector& 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::operator=(const 
		      LOCA::TurningPoint::MooreSpence::ExtendedMultiVector& y)
{
  LOCA::Extended::MultiVector::operator=(y);
  return *this;
}

NOX::Abstract::MultiVector* 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::clone(NOX::CopyType type) const
{
  return new LOCA::TurningPoint::MooreSpence::ExtendedMultiVector(*this, type);
}

NOX::Abstract::MultiVector* 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::clone(int numvecs) const
{
  return 
    new LOCA::TurningPoint::MooreSpence::ExtendedMultiVector(*this, numvecs);
}

NOX::Abstract::MultiVector* 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::subCopy(
					       const vector<int>& index) const
{
  return 
    new LOCA::TurningPoint::MooreSpence::ExtendedMultiVector(*this, index, 
							     false);
}

NOX::Abstract::MultiVector* 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::subView(
					      const vector<int>& index) const
{
  return 
    new LOCA::TurningPoint::MooreSpence::ExtendedMultiVector(*this, index, 
							     true);
}

const NOX::Abstract::MultiVector& 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::getXMultiVec() const
{
  return LOCA::Extended::MultiVector::getMultiVector(0);
}

NOX::Abstract::MultiVector& 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::getXMultiVec()
{
  return LOCA::Extended::MultiVector::getMultiVector(0);
}

const NOX::Abstract::MultiVector& 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::getNullMultiVec() const
{
  return LOCA::Extended::MultiVector::getMultiVector(1);
}

NOX::Abstract::MultiVector& 
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::getNullMultiVec()
{
  return LOCA::Extended::MultiVector::getMultiVector(1);
}

LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::ExtendedMultiVector(
							    int nColumns) :
  LOCA::Extended::MultiVector(nColumns, 2, 1)
{
}

LOCA::Extended::Vector*
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::generateVector(
							int nVecs, 
							int nScalarRows) const
{
  return new LOCA::TurningPoint::MooreSpence::ExtendedVector;
}

LOCA::TurningPoint::MooreSpence::ExtendedVector&
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::getColumn(int i)
{
  return dynamic_cast<LOCA::TurningPoint::MooreSpence::ExtendedVector&>(operator[](i));
}

const LOCA::TurningPoint::MooreSpence::ExtendedVector&
LOCA::TurningPoint::MooreSpence::ExtendedMultiVector::getColumn(int i) const
{
  return dynamic_cast<const LOCA::TurningPoint::MooreSpence::ExtendedVector&>(operator[](i));
}
