// $Id: LOCA_TurningPoint_MooreSpence_ExtendedVector.C,v 1.1.2.1 2005/08/03 23:59:43 etphipp Exp $ 
// $Source: /space/CVS/Trilinos/packages/nox/src-loca/src/LOCA_TurningPoint_MooreSpence_ExtendedVector.C,v $ 

//@HEADER
// ************************************************************************
// 
//                  LOCA Continuation Algorithm Package
//                 Copyright (2005) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//                                                                                 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA                                                                                
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov) or Roger Pawlowski
// (rppawlo@sandia.gov), Sandia National Laboratories.
// 
// ************************************************************************
//@HEADER

#include "LOCA_TurningPoint_MooreSpence_ExtendedVector.H"  // Class definition
#include "LOCA_TurningPoint_MooreSpence_ExtendedMultiVector.H"

LOCA::TurningPoint::MooreSpence::ExtendedVector::ExtendedVector(
					  const NOX::Abstract::Vector& xVec,
					  const NOX::Abstract::Vector& nullVec,
					  double bifParam) :
  LOCA::Extended::Vector(2,1)
{
  setVector(0, xVec);
  setVector(1, nullVec);
  setScalar(0, bifParam);
}

LOCA::TurningPoint::MooreSpence::ExtendedVector::ExtendedVector(
                const LOCA::TurningPoint::MooreSpence::ExtendedVector& source,
		NOX::CopyType type) :
  LOCA::Extended::Vector(source, type)
{
}


LOCA::TurningPoint::MooreSpence::ExtendedVector::~ExtendedVector()
{
}

NOX::Abstract::Vector& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::operator=(
					      const NOX::Abstract::Vector& y)
{
  return operator=(dynamic_cast<const LOCA::TurningPoint::MooreSpence::ExtendedVector&>(y));
}

LOCA::Extended::Vector& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::operator=(
					     const LOCA::Extended::Vector& y)
{
  return operator=(dynamic_cast<const LOCA::TurningPoint::MooreSpence::ExtendedVector&>(y));
}

LOCA::TurningPoint::MooreSpence::ExtendedVector& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::operator=(
                     const LOCA::TurningPoint::MooreSpence::ExtendedVector& y)
{ 
  LOCA::Extended::Vector::operator=(y);
  return *this;
}

NOX::Abstract::Vector* 
LOCA::TurningPoint::MooreSpence::ExtendedVector::clone(
						    NOX::CopyType type) const
{
  return new LOCA::TurningPoint::MooreSpence::ExtendedVector(*this, type);
}

void 
LOCA::TurningPoint::MooreSpence::ExtendedVector::setVec(
					const NOX::Abstract::Vector& xVec,
					const NOX::Abstract::Vector& nullVec,
					double bifPar)
{
  setVector(0, xVec);
  setVector(1, nullVec);
  setScalar(0, bifPar);
}

const NOX::Abstract::Vector& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::getXVec() const
{
  return getVector(0);
}

const NOX::Abstract::Vector& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::getNullVec() const
{
  return getVector(1);
}

double 
LOCA::TurningPoint::MooreSpence::ExtendedVector::getBifParam() const
{
  return getScalar(0);
}

NOX::Abstract::Vector& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::getXVec()
{
  return getVector(0);
}

NOX::Abstract::Vector& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::getNullVec()
{
  return getVector(1);
}

double& 
LOCA::TurningPoint::MooreSpence::ExtendedVector::getBifParam()
{
  return getScalar(0);
}

LOCA::TurningPoint::MooreSpence::ExtendedVector::ExtendedVector() :
  LOCA::Extended::Vector(2,1)
{
}

LOCA::Extended::MultiVector*
LOCA::TurningPoint::MooreSpence::ExtendedVector::generateMultiVector(
							int nColumns, 
							int nVectorRows, 
							int nScalarRows) const
{
  return new LOCA::TurningPoint::MooreSpence::ExtendedMultiVector(nColumns);
}
