// $Id: NOX_Parameter_DirectionConstructor.H,v 1.2 2003/11/25 16:30:10 tgkolda Exp $ 
// $Source: /space/CVS/Trilinos/packages/nox/src/Attic/NOX_Parameter_DirectionConstructor.H,v $ 

//@HEADER
// ************************************************************************
// 
//            NOX: An Object-Oriented Nonlinear Solver Package
//                 Copyright (2002) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//                                                                                 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA                                                                                
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov) or Roger Pawlowski
// (rppawlo@sandia.gov), Sandia National Laboratories.
// 
// ************************************************************************
//@HEADER

#ifndef NOX_PARAMETER_DIRECTIONCONSTRUCTOR_H
#define NOX_PARAMETER_DIRECTIONCONSTRUCTOR_H

#include "NOX_Common.H"		// class data element (string)
#include "NOX_Direction_Generic.H"

namespace NOX {

namespace Parameter {

/*! 
  \brief Support for user-defined directions.

  This object has the ability to be passed via a NOX::Parameter::List
  and to generate new directions.

  It is able to be passed via a NOX::Parameter::List because it is
  derived from NOX::Parameter::Arbitrary class.

  It is used by NOX::Direction::Manager to generate a user-defined
  direction via the newDirection() function.

  The user can write a method that derives from this one or use the
  templated default method, NOX::Parameter::DirectionConstructorT.
*/

class DirectionConstructor : public Arbitrary 
{
  
public:
  
  //! Default Constructor
  DirectionConstructor() {};
  
  //! Destructor
  virtual ~DirectionConstructor() {};

  // derived
  virtual DirectionConstructor* clone() const = 0;

  // derived
  virtual const string& getType() const = 0;

  // derived
  virtual ostream& print(ostream& stream, int indent = 0) const { return stream; };
  
  /*! 
    \brief Construct a new direction and return its pointer.
  */
  virtual NOX::Direction::Generic* newDirection(const NOX::Utils& u, NOX::Parameter::List& p) const = 0;

};

} // namespace Parameter
} // namespace NOX

#endif
