//@HEADER
// ************************************************************************
// 
//            NOX: An Object-Oriented Nonlinear Solver Package
//                 Copyright (2002) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//                                                                                 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA                                                                                
// Questions? Contact Tammy Kolda (tgkolda@sandia.gov) or Roger Pawlowski
// (rppawlo@sandia.gov), Sandia National Laboratories.
// 
// ************************************************************************
//@HEADER

#include "NOX_Parameter_Teuchos2NOX.H"

using namespace NOX::Parameter;

using namespace Teuchos;

List Teuchos2NOX::toNOX(const ParameterList& p) const
{
  List rtn;

  for (ParameterList::ConstIterator i=p.begin(); i!=p.end(); ++i)
    {
      const ParameterEntry& val = p.entry(i);
      const string& name = p.name(i);
      
      if (val.isList())
        {
          rtn.sublist(name) = toNOX(getValue<ParameterList>(val));
        }
      else if (val.isType<int>())
        {
          rtn.setParameter(name, getValue<int>(val));
        }
      else if (val.isType<double>())
        {
          rtn.setParameter(name, getValue<double>(val));
        }
      else if (val.isType<bool>())
        {
          rtn.setParameter(name, getValue<bool>(val));
        }
      else if (val.isType<string>())
        {
          rtn.setParameter(name, getValue<string>(val));
        }
      else
        {
          any data = val.getAny();
          rtn.setParameter(name, AnyPtr(data));
        }
    }

  return rtn;
}



